/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import jp.co.mintjams.generation.base.*;
import jp.co.mintjams.itruth.sql.EntityObject;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderMap;
import jp.morpho.util.StringTokenizer;
import jp.morpho.webapp.servlet.AbstractFilter;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionRequestFactory;
import jp.morpho.webapp.sql.DataSourceFactory;

/**
 * 要求された URL をからカテゴリ ID 、フォーラム ID 、トピック ID を解析する ServletFilter です。
 */
public class ValidateForumIDFilter
	extends AbstractFilter
{
	Logger log = Logger.getLogger(ValidateForumIDFilter.class.getName());

	/**
	 * ValidateForumIDFilter を構築します。
	 */
	public ValidateForumIDFilter()
	{
		super();
	}

	/**
	 * <P>
	 * 要求された URL をからカテゴリ ID 、フォーラム ID 、トピック ID を解析します。
	 * </P>
	 * @param request クライアントがサーブレットに行う要求を含む ServletRequest オブジェクト
	 * @param response サーブレットがクライアントに送信する応答を含む ServletResponse オブジェクト
	 * @param chain フィルタチェーン
	 * @exception IOException フィルタが要求を処理するときに、入出力エラーが検出された場合
	 * @exception ServletException 要求を処理できなかった場合
	 * @see javax.servlet.Filter#doFilter(ServletRequest, ServletResponse, FilterChain)
	 */
	public void doFilter(
		ServletRequest request,
		ServletResponse response,
		FilterChain chain)
		throws IOException, ServletException
	{
		/*
		 * 管理者ページ以外の場合のみ解析します。
		 */
		HttpActionRequest actionRequest = HttpActionRequestFactory.getInstance(getServletContext()).getRequest((HttpServletRequest)request);
		String requestURI = actionRequest.getRequestURI().substring(actionRequest.getContextPath().length());
		int sci = requestURI.indexOf(';');
		if (sci != -1)
		{
			requestURI = requestURI.substring(0, sci);
		}

		/*
		 * URL を解析します。
		 */
		StringTokenizer tokenizer = new StringTokenizer(requestURI, "/");
		String categoryId = null;
		String forumId = null;
		String topicId = null;
		for (int i = 0; tokenizer.hasMoreTokens(); i++)
		{
			switch (i)
			{
				case 1:
					categoryId = tokenizer.nextToken();
					break;
				case 2:
					forumId = tokenizer.nextToken();
					break;
				case 3:
					topicId = tokenizer.nextToken();
					break;
				default:
					tokenizer.nextToken();
					break;
			}
		}

		/*
		 * 取得できた ID によってステータスを切り替えます。
		 */
		if (actionRequest.getStatus().length() == 0)
		{
			if (topicId != null)
			{
				actionRequest.setStatus("post.list");
			}
			else if (forumId != null)
			{
				actionRequest.setStatus("topic.list");
			}
			else if (categoryId != null)
			{
				actionRequest.setStatus("forum.list");
			}
			else
			{
				actionRequest.setStatus("forum.list");
			}
		}

		/*
		 * 取得した ID を保存します。
		 */
		request.setAttribute(ForumGlobals.CATEGORY_ID_KEY, categoryId);
		request.setAttribute(ForumGlobals.FORUM_ID_KEY, forumId);
		request.setAttribute(ForumGlobals.TOPIC_ID_KEY, topicId);

		/*
		 * カテゴリ ID だけでも指定されている場合、フォーラムが存在するか判定します。
		 * 存在しない場合はクライアントにエラーを送信します。
		 */
		if (categoryId != null)
		{
			try
			{
				validateForumID((HttpServletRequest)request);
			}
			catch (Exception e)
			{
				log.log(Level.SEVERE, e.getMessage(), e);
				((HttpServletResponse)response).sendError(HttpServletResponse.SC_NOT_FOUND);
				return;
			}
		}

		chain.doFilter(request, response);
	}

	/**
	 * <P>
	 * 要求されたフォーラムが存在するかどうか判定します。
	 * </P>
	 * @param request クライアントがサーブレットに行う要求を含む ServletRequest オブジェクト
	 * @exception SQLException 要求されたフォーラムが存在しない場合、またはデータベースアクセスエラーが発生した場合
	 */
	private void validateForumID(HttpServletRequest request) throws SQLException
	{
		Connection conn = null;
		try
		{
			/*
			 * DataSourceFactory を作成します。
			 */
			DataSourceFactory dsf = DataSourceFactory.getInstance(request);

			/*
			 * DataSource を取得します。
			 */
			DataSource ds = dsf.getDataSource(Globals.DATASOURCE_NAME);

			/*
			 * Connection を取得します。
			 */
			conn = ds.getConnection();
			conn.setAutoCommit(false);

			/*
			 * フォーラム ID を取得します。
			 */
			String categoryId = (String)request.getAttribute(ForumGlobals.CATEGORY_ID_KEY);
			String forumId = (String)request.getAttribute(ForumGlobals.FORUM_ID_KEY);
			String topicId = (String)request.getAttribute(ForumGlobals.TOPIC_ID_KEY);

			/*
			 * フォーラムを検索します。
			 */
			if (topicId != null)	//トピックが指定されている場合
			{
				EntityObject eoTopics = EntityObject.getInstance("gf_topics", conn);
				PrimitiveHolderMap topics = new ConversionMap();
				topics.putString("categories_id", categoryId);
				topics.putString("forums_id", forumId);
				topics.putString("topics_id", topicId);
				if (eoTopics.findByPrimaryKey(topics) == null)
				{
					throw new SQLException("Topic not found.");
				}
			}
			else if (forumId != null)	//フォーラムが指定されている場合
			{
				EntityObject eoForums = EntityObject.getInstance("gf_forums", conn);
				PrimitiveHolderMap forums = new ConversionMap();
				forums.putString("categories_id", categoryId);
				forums.putString("forums_id", forumId);
				if (eoForums.findByPrimaryKey(forums) == null)
				{
					throw new SQLException("Forum not found.");
				}
			}
			else if (categoryId != null)	//カテゴリが指定されている場合
			{
				EntityObject eoCategories = EntityObject.getInstance("gf_categories", conn);
				PrimitiveHolderMap categories = new ConversionMap();
				categories.putString("categories_id", categoryId);
				if (eoCategories.findByPrimaryKey(categories) == null)
				{
					throw new SQLException("Category not found.");
				}
			}
		}
		catch (SQLException e)
		{
			throw e;
		}
		finally
		{
			/*
			 * Connection をクローズします。
			 */
			try
			{
				conn.close();
			}
			catch (Exception e)
			{
			}
		}
	}

}
