/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.io.FileInputStream;
import java.sql.SQLException;
import java.sql.Types;
import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.Globals;
import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.itruth.sql.EntityObject;
import jp.co.mintjams.itruth.sql.HybridPreparedStatement;
import jp.morpho.io.UploadFile;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * 新規スレッドを登録します。
 */
public class AddTopicHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	public AddTopicHandler()
	{
		super();
	}

	/**
	 * 新規スレッドを登録します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * 現在日時を取得します。
		 */
		java.util.Date now = new java.util.Date();

		/*
		 * フォームデータを取得します。
		 */
		PrimitiveHolderMap form = getParameterMap();

		try
		{
			long topicsId = 0;

			int retryCount = 0;
			do
			{
				try
				{
					/*
					 * トピックIDを作成します。
					 */
					topicsId = (new java.util.Date()).getTime();

					/*
					 * topics にデータを登録します。
					 */
					EntityObject eoTopics = EntityObject.getInstance("gf_topics", getConnection());
					//
					PrimitiveHolderMap topics = new ConversionMap();
					topics.putString("categories_id", form.getString("categories_id"));
					topics.putString("forums_id", form.getString("forums_id"));
					topics.putLong("topics_id", topicsId);
					topics.putString("topics_status", form.getString("topics_status"));
					topics.putLong("topics_last_posts_id", topicsId);
					topics.putString("users_id", (String)request.getAttribute(Globals.USER_ID_KEY));
					topics.putDate("date_added", now);
					//
					eoTopics.insert(topics);
				}
				catch (SQLException se)
				{
					if (retryCount < 3)
					{
						retryCount++;
						continue;
					}
					throw se;
				}
			}
			while (false);

			/*
			 * posts にデータを登録します。
			 */
			EntityObject eoPosts = EntityObject.getInstance("gf_posts", getConnection());
			//
			PrimitiveHolderMap posts = new ConversionMap();
			posts.putString("categories_id", form.getString("categories_id"));
			posts.putString("forums_id", form.getString("forums_id"));
			posts.putLong("topics_id", topicsId);
			posts.putLong("posts_id", topicsId);
			posts.putString("posts_title", form.getString("posts_title"));
			posts.putString("posts_message", form.getString("posts_message"));
			posts.putString("users_id", (String)request.getAttribute(Globals.USER_ID_KEY));
			posts.putDate("date_added", now);
			posts.putDate("date_lastmodified", now);
			//
			eoPosts.insert(posts);

			/*
			 * アップロードされたファイルを取得します。
			 */
			UploadFile upload = (UploadFile)form.get("posts_file");
			String path = (upload != null) ? upload.getUploadPath() : null;

			if (path != null && path.length() > 0)
			{
				/*
				 * ファイル名を作成します。
				 */
				String fileName = null;
				{
					if (path.indexOf("\\") != -1)
					{
						fileName = path.substring(path.lastIndexOf("\\") + 1);
					}
					else if (path.indexOf("/") != -1)
					{
						fileName = path.substring(path.lastIndexOf("/") + 1);
					}
					else
					{
						fileName = path;
					}
				}

				/*
				 * postfiles にデータを登録します。
				 */
				EntityObject eoFiles = EntityObject.getInstance("gf_postfiles", getConnection());
				//
				PrimitiveHolderMap files = new ConversionMap();
				files.putString("categories_id", form.getString("categories_id"));
				files.putString("forums_id", form.getString("forums_id"));
				files.putLong("topics_id", topicsId);
				files.putLong("posts_id", topicsId);
				files.putString("postfiles_file_name", fileName);
				files.putString("postfiles_file_contenttype", upload.getContentType());
				files.putLong("postfiles_file_size", upload.length());
				files.putInputStream("postfiles_data", new FileInputStream(upload));
				//
				eoFiles.insert(files);
			}

			HybridPreparedStatement stmt = null;

			/*
			 * forums を更新します。
			 */
			try
			{
				StringBuffer sql = new StringBuffer();
				sql.append("UPDATE gf_forums SET forums_last_topics_id = ?");
				sql.append(" WHERE categories_id = ? AND forums_id = ?");
				stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
				stmt.setParameter(1, new Long(topicsId), Types.DECIMAL);
				stmt.setParameter(2, form.getString("categories_id"), Types.VARCHAR);
				stmt.setParameter(3, form.getString("forums_id"), Types.VARCHAR);
				stmt.executeUpdate();
			}
			catch (SQLException e)
			{
				throw e;
			}
			finally
			{
				try
				{
					stmt.close();
				}
				catch (Exception e)
				{
				}
			}

			/*
			 * コミットします。
			 */
			getConnection().commit();

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("topic.new.success");
		}
		catch (Exception e)
		{
			/*
			 * 例外トレースを出力します
			 */
			log.log(Level.WARNING, "例外が発生しました", e);

			/*
			 * ロールバックします。
			 */
			try
			{
				getConnection().rollback();
			}
			catch (Exception sqle)
			{
			}

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("topic.new.failure");
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

}
