/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.itruth.sql.EntityObject;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * 基本情報を登録します。
 */
public class ChangeVarsHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	public ChangeVarsHandler()
	{
		super();
	}

	/**
	 * 基本情報を登録します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * フォームデータを取得します。
		 */
		PrimitiveHolderMap form = getParameterMap();

		try
		{
			String key = "";

			//フォーラム タイトル
			key = "title";
			updateVar(key, form.getString(key));

			//フォーラム サブタイトル
			key = "subtitle";
			updateVar(key, form.getString(key));

			//検索キーワードの最大値
			key = "max_search_keywords";
			updateVar(key, form.getString(key));

			//検索結果の最大表示件数
			key = "max_search_results";
			updateVar(key, form.getString(key));

			//トピックの１ページの表示件数
			key = "max_topics_rows";
			updateVar(key, form.getString(key));

			//投稿の１ページの表示件数
			key = "max_posts_rows";
			updateVar(key, form.getString(key));

			/*
			 * コミットします。
			 */
			getConnection().commit();

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("vars.edit.success");
		}
		catch (Exception e)
		{
			/*
			 * 例外トレースを出力します
			 */
			log.log(Level.WARNING, "例外が発生しました", e);

			/*
			 * ロールバックします。
			 */
			try
			{
				getConnection().rollback();
			}
			catch (Exception sqle)
			{
			}

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("vars.edit.failure");
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

	private void updateVar(String key, String val) throws Exception
	{
		/*
		 * 列のマップを作成します。
		 */
		PrimitiveHolderMap pkeys = new ConversionMap();
		pkeys.putString("vars_key", key);

		/*
		 * vars を削除します。
		 */
		EntityObject eoVars = EntityObject.getInstance("gf_vars", getConnection());
		eoVars.deleteByPrimaryKey(pkeys);

		if (val.length() > 0)
		{
			/*
			 * vars にデータを登録します。
			 */
			//
			PrimitiveHolderMap vars = new ConversionMap();
			vars.putString("vars_key", key);
			vars.putString("vars_value", val);
			//
			eoVars.insert(vars);
		}
	}
}
