/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.sql.SQLException;
import java.sql.Types;
import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.itruth.sql.EntityObject;
import jp.co.mintjams.itruth.sql.HybridPreparedStatement;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * フォーラムを削除します。
 */
public class DeleteForumHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	public DeleteForumHandler()
	{
		super();
	}

	/**
	 * フォーラムを削除します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * フォームデータを取得します。
		 */
		ConversionMap form = getParameterMap();

		try
		{
			/*
			 * カテゴリID
			 */
			 String categoriesId = form.getString("categories_id");

			/*
			 * フォーラムID
			 */
			 String forumsId = form.getString("forums_id");

			/*
			 * フォーラムを削除します。
			 */
			EntityObject eoForums = EntityObject.getInstance("gf_forums", getConnection());
			PrimitiveHolderMap forums = new ConversionMap();
			forums.putString("categories_id", categoriesId);
			forums.putString("forums_id", forumsId);
			eoForums.deleteByPrimaryKey(forums);

			HybridPreparedStatement stmt = null;

			/*
			 * すべてのトピックを削除します。
			 */
			try
			{
				StringBuffer sql = new StringBuffer();
				sql.append("DELETE FROM gf_topics WHERE categories_id = ? AND forums_id = ?");
				stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
				stmt.setParameter(1, categoriesId, Types.VARCHAR);
				stmt.setParameter(2, forumsId, Types.VARCHAR);
				stmt.executeUpdate();
			}
			catch (SQLException e)
			{
				throw e;
			}
			finally
			{
				try
				{
					stmt.close();
				}
				catch (Exception e)
				{
				}
			}

			/*
			 * すべての投稿を削除します。
			 */
			try
			{
				StringBuffer sql = new StringBuffer();
				sql.append("DELETE FROM gf_posts WHERE categories_id = ? AND forums_id = ?");
				stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
				stmt.setParameter(1, categoriesId, Types.VARCHAR);
				stmt.setParameter(2, forumsId, Types.VARCHAR);
				stmt.executeUpdate();
			}
			catch (SQLException e)
			{
				throw e;
			}
			finally
			{
				try
				{
					stmt.close();
				}
				catch (Exception e)
				{
				}
			}

			/*
			 * すべての添付ファイルを削除します。
			 */
			try
			{
				StringBuffer sql = new StringBuffer();
				sql.append("DELETE FROM gf_postfiles WHERE categories_id = ? AND forums_id = ?");
				stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
				stmt.setParameter(1, categoriesId, Types.VARCHAR);
				stmt.setParameter(2, forumsId, Types.VARCHAR);
				stmt.executeUpdate();
			}
			catch (SQLException e)
			{
				throw e;
			}
			finally
			{
				try
				{
					stmt.close();
				}
				catch (Exception e)
				{
				}
			}

			/*
			 * コミットします。
			 */
			getConnection().commit();

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("forum.delete.success");
		}
		catch (Exception e)
		{
			log.log(Level.WARNING, "例外が発生しました", e);

			/*
			 * ロールバックします。
			 */
			try
			{
				getConnection().rollback();
			}
			catch (Exception e1)
			{
			}

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("forum.delete.failure");

			throw new ActionHandlerException(e.getMessage());
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

}
