/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.sql.Types;
import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.itruth.sql.HybridPreparedStatement;
import jp.morpho.util.ConversionMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * トピックを削除します。
 */
public class DeleteTopicsHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.todo");

	public DeleteTopicsHandler()
	{
		super();
	}

	/**
	 * トピックを削除します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * 現在日時を取得します。
		 */
		java.util.Date now = new java.util.Date();

		/*
		 * フォームデータを取得します。
		 */
		ConversionMap form = getParameterMap();

		HybridPreparedStatement stmtTopics = null;
		HybridPreparedStatement stmtPostFiles = null;
		HybridPreparedStatement stmtPosts = null;
		try
		{
			/*
			 * Topic を削除する Statement を作成します。
			 */
			{
				StringBuffer sql = new StringBuffer();
				sql.append("UPDATE gf_topics SET date_deleted = ?");
				sql.append(" WHERE categories_id = ? AND forums_id = ? AND topics_id = ? AND date_deleted IS NULL");
				stmtTopics = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
			}
			/*
			 * 投稿を削除する Statement を作成します。
			 */
			{
				StringBuffer sql = new StringBuffer();
				sql.append("UPDATE gf_posts SET date_deleted = ?");
				sql.append(" WHERE categories_id = ? AND forums_id = ? AND topics_id = ? AND date_deleted IS NULL");
				stmtPosts = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
			}
			/*
			 * 添付ファイルを削除する Statement を作成します。
			 */
			{
				StringBuffer sql = new StringBuffer();
				sql.append("UPDATE gf_postfiles SET date_deleted = ?");
				sql.append(" WHERE categories_id = ? AND forums_id = ? AND topics_id = ? AND date_deleted IS NULL");
				stmtPostFiles = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
			}

			/*
			 * カテゴリID
			 */
			 String categoriesId = form.getString("categories_id");

			/*
			 * フォーラムID
			 */
			 String forumsId = form.getString("forums_id");

			/*
			 * topics_id の配列を作成します。
			 */
			String[] keys = null;
			if (form.isArray("topics_ids"))
			{
				keys = (String[])form.get("topics_ids");
			}
			else
			{
				keys = new String[1];
				keys[0] = form.getString("topics_ids");
			}

			/*
			 * データを削除します。
			 */
			for (int i = 0; i < keys.length; i++)
			{
				/*
				 * Topic を削除します。
				 */
				stmtTopics.setParameter(1, now, Types.TIMESTAMP);
				stmtTopics.setParameter(2, categoriesId, Types.VARCHAR);
				stmtTopics.setParameter(3, forumsId, Types.VARCHAR);
				stmtTopics.setParameter(4, keys[i], Types.DECIMAL);
				stmtTopics.executeUpdate();

				/*
				 * 投稿を削除します。
				 */
				stmtPosts.setParameter(1, now, Types.TIMESTAMP);
				stmtPosts.setParameter(2, categoriesId, Types.VARCHAR);
				stmtPosts.setParameter(3, forumsId, Types.VARCHAR);
				stmtPosts.setParameter(4, keys[i], Types.DECIMAL);
				stmtPosts.executeUpdate();

				/*
				 * 添付ファイルを削除します。
				 */
				stmtPostFiles.setParameter(1, now, Types.TIMESTAMP);
				stmtPostFiles.setParameter(2, categoriesId, Types.VARCHAR);
				stmtPostFiles.setParameter(3, forumsId, Types.VARCHAR);
				stmtPostFiles.setParameter(4, keys[i], Types.DECIMAL);
				stmtPostFiles.executeUpdate();
			}

			/*
			 * コミットします。
			 */
			getConnection().commit();

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("topics.delete.success");
		}
		catch (Exception e)
		{
			/*
			 * ロールバックします。
			 */
			try
			{
				getConnection().rollback();
			}
			catch (Exception sqle)
			{
			}

			/*
			 * 例外トレースを出力します
			 */
			log.log(Level.WARNING, "例外が発生しました", e);

			/*
			 * ステータスを更新します。
			 */
			request.setStatus("topics.delete.failure");
		}
		finally
		{
			/*
			 * Statement をクローズします。
			 */
			try
			{
				stmtTopics.close();
			}
			catch (Exception e)
			{
			}
			try
			{
				stmtPosts.close();
			}
			catch (Exception e)
			{
			}
			try
			{
				stmtPostFiles.close();
			}
			catch (Exception e)
			{
			}
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

}
