/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.generation.forum.ForumGlobals;
import jp.co.mintjams.itruth.sql.HybridPreparedStatement;
import jp.co.mintjams.itruth.sql.ResultSetCache;
import jp.morpho.util.ConversionMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * カテゴリIDの数だけ次のハンドラを呼び出します。
 * SearchCategoriesHandler の後にチェーンに登録することにより、複数カテゴリに対するフォーラムリストの取得に対応します。
 */
public class ForSearchCategoryForumsHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	/**
	 * ForSearchCategoryForumsHandler を作成します。
	 */
	public ForSearchCategoryForumsHandler()
	{
		super();
	}

	/**
	 * カテゴリIDの数だけ次のハンドラを呼び出します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * フォームデータを取得します。
		 */
		ConversionMap form = getParameterMap();

		HybridPreparedStatement stmt = null;
		ResultSet rs = null;

		try
		{
			StringBuffer sql = new StringBuffer();
			sql.append("SELECT");
			sql.append(" f.categories_id AS categories_id");
			sql.append(",f.forums_id AS forums_id");
			sql.append(",f.forums_name AS forums_name");
			sql.append(",f.forums_contents AS forums_contents");
			sql.append(",f.date_added AS forums_date_added");
			sql.append(",f.users_id AS moderators_id");
			sql.append(",mu.users_name AS moderators_name");
			sql.append(",tc.counts AS topics_count");
			sql.append(",(tc.counts + tc.replies) AS posts_count");
			sql.append(",f.forums_last_topics_id AS last_topics_id");
			sql.append(",t.topics_last_posts_id AS last_posts_id");
			sql.append(",p.date_added AS date_lastposted");
			sql.append(",p.users_id AS lastposters_id");
			sql.append(",pu.users_name AS lastposters_name");
			sql.append(",t.topics_status AS topics_status");
			sql.append(" FROM");
			sql.append(" gf_forums f");
			sql.append(" LEFT OUTER JOIN gb_users mu ON (f.users_id = mu.users_id)");
			sql.append(" LEFT OUTER JOIN (");
			sql.append("    SELECT");
			sql.append("     categories_id AS categories_id");
			sql.append("    ,forums_id AS forums_id");
			sql.append("    ,COUNT(*) AS counts");
			sql.append("    ,SUM(topics_replies) AS replies");
			sql.append("     FROM");
			sql.append("     gf_topics");
			sql.append("     GROUP BY");
			sql.append("     categories_id");
			sql.append("    ,forums_id) tc ON (f.categories_id = tc.categories_id AND f.forums_id = tc.forums_id)");
			sql.append(" LEFT OUTER JOIN gf_topics t ON (f.categories_id = t.categories_id AND f.forums_id = t.forums_id AND f.forums_last_topics_id = t.topics_id)");
			sql.append(" LEFT OUTER JOIN gf_posts p ON (t.categories_id = p.categories_id AND t.forums_id = p.forums_id AND t.topics_id = p.topics_id AND t.topics_last_posts_id = p.posts_id)");
			sql.append(" LEFT OUTER JOIN gb_users pu ON (p.users_id = pu.users_id)");
			sql.append(" WHERE f.categories_id = ?");
			sql.append(" ORDER BY forums_id");

			stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));

			String categoryId = (String)request.getAttribute(ForumGlobals.CATEGORY_ID_KEY);

			/*
			 * カテゴリIDの数だけフォーラムを検索します。
			 */
			List categories = (List)request.getAttribute("categories");
			for (int i = 0; categories != null && i < categories.size(); i++)
			{
				ConversionMap category = ConversionMap.valueOf((Map)categories.get(i));

				/*
				 * カテゴリIDが指定されていない場合は、全てを検索します。
				 * カテゴリIDが指定されている場合は、指定されたカテゴリIDのみを検索します。
				 */
				if (categoryId == null || (categoryId != null && categoryId.equals(category.getString("categories_id"))))
				{
					stmt.setParameter(1, category.getString("categories_id"), Types.VARCHAR);
					rs = stmt.executeQuery();
					ResultSetCache rsc = new ResultSetCache(rs, 0, 0);

					/*
					 * マップに保存します。
					 */
					((Map)categories.get(i)).put("forums", rsc.toList());

					try
					{
						rs.close();
					}
					catch (Exception e)
					{
					}
				}
			}

		}
		catch (SQLException e)
		{
			log.log(Level.WARNING, "例外が発生しました", e);
			throw new ActionHandlerException(e.getMessage());
		}
		finally
		{
			try
			{
				rs.close();
			}
			catch (Exception e)
			{
			}
			try
			{
				stmt.close();
			}
			catch (Exception e)
			{
			}
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

}
