/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.sql.SQLException;
import java.util.Map;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.itruth.sql.EntityObject;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * フォーラム変数情報を取得するための基本クラスです。
 */
public abstract class InitVarHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	/**
	 * vars にアクセスする EntityObject。
	 * doActionHandler を呼び出したときに設定されます。
	 */
	private EntityObject eoVars = null;

	public InitVarHandler()
	{
		super();
	}

	/**
	 * 変数テーブルから指定された key に対応する変数の値を返します。
	 *
	 * @param key 変数のキー
	 * @return key に対応する変数の値。変数テーブルに対応する値がない場合は null
	 */
	protected String getVarsValue(String key) throws SQLException
	{
		PrimitiveHolderMap pkeys = new ConversionMap();
		pkeys.putString("vars_key", key);

		/*
		 * データを取得します。
		 */
		Map vars = eoVars.findByPrimaryKey(pkeys);

		/*
		 * フォームデータにデータを設定します。
		 */
		if (vars != null)
		{
			return ConversionMap.valueOf(vars).getString("vars_value");
		}
		return null;
	}

	/**
	 * この InitVarHandler を初期化します。
	 * このメソッドをオーバーライドし、最初に super.doActionHandler(request, response, chain) を呼び出してください。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * vars にアクセスする EntityObject を作成します。
		 */
		eoVars = EntityObject.getInstance("gf_vars", getConnection());
	}

}
