/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.itruth.sql.HybridPreparedStatement;
import jp.co.mintjams.itruth.sql.ResultSetCache;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.StringTokenizer;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * 投稿を指定された条件で検索します。
 */
public class SearchForumsPostsHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	public SearchForumsPostsHandler()
	{
		super();
	}

	/**
	 * 投稿を指定された条件で検索します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * フォームデータを取得します。
		 */
		ConversionMap form = getParameterMap();

		/*
		 * 検索キーワードを取得します。
		 */
		StringTokenizer token = new StringTokenizer(form.getString("keywords"), " ");
		String[] keywords = token.getTokens();
		int len = (keywords.length < form.getInt("max_search_keywords")) ? keywords.length : form.getInt("max_search_keywords");
		form.putInt("keywords_count", keywords.length);

		StringBuffer whereTitle = new StringBuffer();
		StringBuffer whereMessage = new StringBuffer();
		for (int i = 0; i < len; i++)
		{
			if (i > 0)
			{
				whereTitle.append(" AND");
				whereMessage.append(" AND");
			}
			whereTitle.append(" p.posts_title LIKE ?");
			whereMessage.append(" p.posts_message LIKE ?");
		}

		/*
		 * 投稿を検索します。
		 */
		StringBuffer sql = new StringBuffer();
		sql.append("SELECT");
		sql.append(" p.categories_id AS categories_id");
		sql.append(",p.forums_id AS forums_id");
		sql.append(",p.topics_id AS topics_id");
		sql.append(",p.posts_id AS posts_id");
		sql.append(",p.posts_title AS posts_title");
		sql.append(",p.date_added AS date_added");
		sql.append(",f.forums_name AS forums_name");
		sql.append(",t.topics_views AS topics_views");
		sql.append(",t.topics_replies AS topics_replies");
		sql.append(",p.users_id AS posters_id");
		sql.append(",m.users_name AS posters_name");
		sql.append(" FROM gf_posts p");
		sql.append(" LEFT OUTER JOIN gb_users m ON (p.users_id = m.users_id)");
		sql.append(" LEFT OUTER JOIN gf_forums f ON (p.categories_id = f.categories_id AND p.forums_id = f.forums_id)");
		sql.append(" LEFT OUTER JOIN gf_topics t ON (p.categories_id = t.categories_id AND p.forums_id = t.forums_id AND p.topics_id = t.topics_id)");
		sql.append(" WHERE p.date_deleted IS NULL");
		sql.append(" AND ((").append(whereTitle).append(") OR (").append(whereMessage).append("))");
		sql.append(" ORDER BY date_added DESC");

		HybridPreparedStatement stmt = null;
		ResultSet rs = null;
		try
		{
			stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
			for (int i = 0; i < len; i++)
			{
				String keyword = "%" + keywords[i] + "%";
				stmt.setVarchar((i + 1), keyword);
				stmt.setVarchar((i + 1) + len, keyword);
			}
			rs = stmt.executeQuery();
			ResultSetCache rsc = new ResultSetCache(rs, 1, form.getInt("max_search_results"));

			form.putInt("max_rows", rsc.getMaxRows());
			request.setAttribute("results", rsc.toList());
		}
		catch (SQLException e)
		{
			log.log(Level.WARNING, "例外が発生しました", e);
			throw new ActionHandlerException(e.getMessage());
		}
		finally
		{
			try
			{
				rs.close();
			}
			catch (Exception e)
			{
			}
			try
			{
				stmt.close();
			}
			catch (Exception e)
			{
			}
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

}
