/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.itruth.sql.HybridPreparedStatement;
import jp.co.mintjams.itruth.sql.ResultSetCache;
import jp.morpho.util.ConversionMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * 投稿を検索します。
 */
public class SearchPostsHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	public SearchPostsHandler()
	{
		super();
	}

	/**
	 * 投稿を検索します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * フォームデータを取得します。
		 */
		ConversionMap form = getParameterMap();

		/*
		 * ページを取得します。
		 */
		int pages = form.getInt("page_no");
		int startNo = 0;
		int endNo = 0;

		if (pages <= 1)
		{
			form.putInt("page_no", 1);
			startNo = 1;
			endNo = form.getInt("max_posts_rows");
		}
		else
		{
			startNo = ((pages - 1) * form.getInt("max_posts_rows")) + 1;
			endNo = pages * form.getInt("max_posts_rows");
		}

		String sort = "ASC";
		if (form.getString("sort").length() > 0)
		{
			sort = form.getString("sort");
		}

		/*
		 * 投稿を検索します。
		 */
		StringBuffer sql = new StringBuffer();
		sql.append("SELECT");
		sql.append(" p.posts_id AS posts_id");
		sql.append(",p.posts_title AS posts_title");
		sql.append(",p.posts_message AS posts_message");
		sql.append(",p.users_id AS posters_id");
		sql.append(",p.date_added AS date_added");
		sql.append(",p.date_deleted AS date_deleted");
		sql.append(",m.users_name AS posters_name");
		sql.append(",m.users_avatar AS posters_avatar");
		sql.append(",m.date_added AS posters_added");
		sql.append(",pf.postfiles_file_size AS postfiles_file_size");
		sql.append(" FROM gf_posts p");
		sql.append(" LEFT OUTER JOIN gb_users m ON (p.users_id = m.users_id)");
		sql.append(" LEFT OUTER JOIN gf_postfiles pf ON (p.categories_id = pf.categories_id AND p.forums_id = pf.forums_id AND p.topics_id = pf.topics_id AND p.posts_id = pf.posts_id)");
		sql.append(" WHERE p.categories_id = ?");
		sql.append(" AND p.forums_id = ?");
		sql.append(" AND p.topics_id = ?");
		sql.append(" ORDER BY posts_id ").append(sort);

		HybridPreparedStatement stmt = null;
		ResultSet rs = null;
		try
		{
			stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
			stmt.setParameter(1, form.getString("categories_id"), Types.VARCHAR);
			stmt.setParameter(2, form.getString("forums_id"), Types.VARCHAR);
			stmt.setParameter(3, form.getString("topics_id"), Types.DECIMAL);
			rs = stmt.executeQuery();
			ResultSetCache rsc = new ResultSetCache(rs, startNo, endNo);

			form.putInt("max_rows", rsc.getMaxRows());
			request.setAttribute("posts", rsc.toList());
		}
		catch (SQLException e)
		{
			log.log(Level.WARNING, "例外が発生しました", e);
			throw new ActionHandlerException(e.getMessage());
		}
		finally
		{
			try
			{
				rs.close();
			}
			catch (Exception e)
			{
			}
			try
			{
				stmt.close();
			}
			catch (Exception e)
			{
			}
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

}
