/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.logging.Level;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.generation.base.util.DateUtils;
import jp.co.mintjams.itruth.sql.HybridPreparedStatement;
import jp.co.mintjams.itruth.sql.ResultSetCache;
import jp.morpho.util.ConversionMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * トピックを検索します。
 */
public class SearchTopicsHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	public SearchTopicsHandler()
	{
		super();
	}

	/**
	 * トピックを検索します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * フォームデータを取得します。
		 */
		ConversionMap form = getParameterMap();

		/*
		 * ページを取得します。
		 */
		int pages = form.getInt("page_no");
		int startNo = 0;
		int endNo = 0;

		if (pages <= 1)
		{
			form.putInt("page_no", 1);
			startNo = 1;
			endNo = form.getInt("max_topics_rows");
		}
		else
		{
			startNo = ((pages - 1) * form.getInt("max_topics_rows")) + 1;
			endNo = pages * form.getInt("max_topics_rows");
		}

		String sort = form.getString("sort");
		String sortItem = form.getString("sort_item");
		int searchTerm = form.getInt("search_term");

		/*
		 * トピックを検索します。
		 */
		StringBuffer sql = new StringBuffer();
		sql.append("SELECT");
		sql.append(" t.topics_id AS topics_id");
		sql.append(",t.topics_status AS topics_status");
		sql.append(",t.topics_views AS topics_views");
		sql.append(",t.topics_replies AS topics_replies");
		sql.append(",t.users_id AS posters_id");
		sql.append(",fu.users_name AS posters_name");
		sql.append(",fp.posts_title AS posts_title");
		sql.append(",lp.date_added AS date_lastposted");
		sql.append(",lp.users_id AS lastposters_id");
		sql.append(",lu.users_name AS lastposters_name");
		sql.append(" FROM");
		sql.append(" gf_topics t");
		sql.append(" LEFT OUTER JOIN gb_users fu ON (t.users_id = fu.users_id)");
		sql.append(" LEFT OUTER JOIN gf_posts fp ON (t.categories_id = fp.categories_id AND t.forums_id = fp.forums_id AND t.topics_id = fp.topics_id AND t.topics_id = fp.posts_id)");
		sql.append(" LEFT OUTER JOIN gf_posts lp ON (t.categories_id = lp.categories_id AND t.forums_id = lp.forums_id AND t.topics_id = lp.topics_id AND t.topics_last_posts_id = lp.posts_id)");
		sql.append(" LEFT OUTER JOIN gb_users lu ON (lp.users_id = lu.users_id)");
		sql.append(" WHERE t.categories_id = ?");
		sql.append(" AND t.forums_id = ?");
		sql.append(" AND t.date_deleted IS NULL");
		if (searchTerm > 0)
		{
			sql.append(" AND lp.date_added >= ?");
		}
		sql.append(" ORDER BY ").append(sortItem).append(" ").append(sort);

		HybridPreparedStatement stmt = null;
		ResultSet rs = null;
		try
		{
			stmt = new HybridPreparedStatement(getConnection().prepareStatement(sql.toString()));
			stmt.setParameter(1, form.getString("categories_id"), Types.VARCHAR);
			stmt.setParameter(2, form.getString("forums_id"), Types.VARCHAR);
			if (searchTerm > 0)
			{
				java.util.Date searchDate = DateUtils.add(DateUtils.clearTime(new java.util.Date()), -searchTerm);
				stmt.setParameter(3, searchDate, Types.TIMESTAMP);
			}
			rs = stmt.executeQuery();
			ResultSetCache rsc = new ResultSetCache(rs, startNo, endNo);

			form.putInt("max_rows", rsc.getMaxRows());
			request.setAttribute("topics", rsc.toList());
		}
		catch (SQLException e)
		{
			log.log(Level.WARNING, "例外が発生しました", e);
			throw new ActionHandlerException(e.getMessage());
		}
		finally
		{
			try
			{
				rs.close();
			}
			catch (Exception e)
			{
			}
			try
			{
				stmt.close();
			}
			catch (Exception e)
			{
			}
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}

}
