/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.forum.action;

import java.util.Map;
import java.util.logging.Logger;

import jp.co.mintjams.generation.base.Globals;
import jp.co.mintjams.generation.base.action.DatabaseAccessHandler;
import jp.co.mintjams.generation.base.util.DateUtils;
import jp.co.mintjams.itruth.sql.EntityObject;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderMap;
import jp.morpho.webapp.action.ActionHandlerChain;
import jp.morpho.webapp.action.ActionHandlerException;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionResponse;

/**
 * 訪問日時を取得・更新します。
 */
public class VisitHandler extends DatabaseAccessHandler
{
	Logger log = Logger.getLogger("jp.co.mintjams.generation.forum");

	public VisitHandler()
	{
		super();
	}

	/**
	 * 訪問日時を取得・更新します。
	 *
	 * @see jp.morpho.webapp.action.ActionHandler#doActionHandler(jp.morpho.webapp.servlet.http.HttpActionRequest, jp.morpho.webapp.servlet.http.HttpActionResponse, jp.morpho.webapp.action.ActionHandlerChain)
	 */
	public void doActionHandler(
		HttpActionRequest request,
		HttpActionResponse response,
		ActionHandlerChain chain)
		throws ActionHandlerException
	{
		super.doActionHandler(request, response, chain);

		/*
		 * 現在日時を取得します。
		 */
		java.util.Date now = new java.util.Date();

		/*
		 * フォームデータを取得します。
		 */
		PrimitiveHolderMap form = getParameterMap();

		try
		{
			String userId = (String)request.getAttribute(Globals.USER_ID_KEY);

			/*
			 * visits にアクセスする EntityObject を作成します。
			 */
			EntityObject eoVisits = EntityObject.getInstance("gf_visits", getConnection());

			/*
			 * 列のマップを作成します。
			 */
			PrimitiveHolderMap pkeys = new ConversionMap();
			pkeys.putString("users_id", userId);

			/*
			 * データを取得します。
			 */
			Map visits = eoVisits.findByPrimaryKey(pkeys);

			/*
			 * フォームデータにデータを設定します。
			 */
			if (visits != null)
			{
				//2回目以降の訪問
				PrimitiveHolderMap lastVisits = ConversionMap.valueOf(visits);
				java.util.Date visitedDate = DateUtils.clearTime(lastVisits.getDate("date_visited"));
				if (DateUtils.clearTime(now).after(visitedDate))
				{
					//
					PrimitiveHolderMap newestVisits = new ConversionMap();
					newestVisits.put("users_id", userId);
					newestVisits.put("date_visited", now);
					newestVisits.put("date_lastvisited", lastVisits.getDate("date_visited"));
					//
					eoVisits.updateByPrimaryKey(newestVisits);

					form.put("date_visited", newestVisits.get("date_visited"));
					form.put("date_lastvisited", newestVisits.get("date_lastvisited"));
				}
				else
				{
					form.put("date_visited", visits.get("date_visited"));
					form.put("date_lastvisited", visits.get("date_lastvisited"));
				}
			}
			else
			{
				//初めての訪問
				PrimitiveHolderMap newestVisits = new ConversionMap();
				newestVisits.put("users_id", userId);
				newestVisits.put("date_visited", now);
				newestVisits.put("date_lastvisited", now);
				//
				eoVisits.insert(newestVisits);

				form.put("date_visited", newestVisits.get("date_visited"));
				form.put("date_lastvisited", newestVisits.get("date_lastvisited"));
			}

			/*
			 * コミットします。
			 */
			getConnection().commit();
		}
		catch (Exception e)
		{
			e.printStackTrace();
			log.warning(e.getMessage());

			/*
			 * ロールバックします。
			 */
			try
			{
				getConnection().rollback();
			}
			catch (Exception sqle)
			{
			}
		}

		/*
		 * 次の ActionHandler を呼び出します。
		 */
		chain.doActionHandler(request, response);
	}
}
